/*****************************************************************************
 * DMC_SCB_PRIO_Test_Core0.c
 *****************************************************************************/

/*This code illustrates the bandwidth limit feature. It configures an MDMA from DMC and measures
   the throughput by polling the DMA status bit.
 */

#include <ccblkfn.h>
#include "adi_initialize.h"
#include "Bandwidth_Limit_Core0.h"
#include "ddr_init.h"
#include <cdefBF609.h>
#include <defBF609.h>
#include "time.h"
#include "stdio.h"

#pragma align 32
section("L1_data_a") unsigned char ucIntDestBuff[MAX_SIZE];  //Destination buffer in internal memory
section("sdram0_bank0") unsigned char ucExtSrcBuff[MAX_SIZE]; //Source buffer in external (DDR2) memory

//Global variables
volatile clock_t clock_start;
volatile clock_t clock_stop_mdma0;
volatile clock_t cycles_mdma0;
float throughput_mdma0;
int DMA_Done_MDMA0=0;

extern void DDR2_Init(void);

void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg);

//Macro to configure MDMA

#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;

//Macro to wait for DMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));

//Macro to disable the MDMA
#define DISABLE_DMAs(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\

int main(void)
{
	//DMA buffer size = 16 K bytes
	unsigned int uiBytes=16384;

	//Initialize DMC
	DDR2_Init();

	//Initialize iterrupts and handlers
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA0_DST, MDMA0_Interrupt_Handler, NULL, true);

	//Configure MDMA 21-22 channels to read from DMC and
	MDMA_CONFIG(21, SOURCE_DMA_CFG,DDR2_BANK0_START_ADDRESS, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,22,DEST_DMA_CFG, ucIntDestBuff, DEST_MSIZE, uiBytes/DEST_MSIZE);

	//Configure bandwidth limit register here (BWLCNT = SCLK speed in MHz*MSIZE in bytes/Bandwidth required in MB/s)
	*pREG_DMA21_BWLCNT=128;	//For MSIZE=8 and expected BW = 7.7 MB/s

	//Enable DMAs now
	*pREG_DMA22_CFG |= 0x1;
	*pREG_DMA21_CFG |= 0x1;


	//Start the cycle count
	clock_start = clock();

	//Wait till DMA done
	while(DMA_Done_MDMA0==0);

	//Calculate the number of cycles to complete MDMA
	cycles_mdma0=clock_stop_mdma0-clock_start;

	//Disable the DMAs here
	DISABLE_DMAs(21,22);

	//Calculate and print the measured throughput
	throughput_mdma0=(float)(uiBytes*CCLK_SPEED/cycles_mdma0);
	printf("\n\n No of bytes=%d, No ofd cycles for MDMA0 =%d\n\n ",uiBytes,cycles_mdma0);
	printf("Throughput MDMA0=%f MB/s\n\n",throughput_mdma0);

	return 0;
}


void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count
	clock_stop_mdma0 = clock();

	*pREG_DMA22_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA0 = 1;
}


