/*
 *  (C) Copyright 2012 - Analog Devices, Inc.  All rights reserved.
 * 	File name : DDR2Init.h
 * 	Purpose: Includes the macros, variables and function declarations for DDR2Init.C
 * 	Hardware used: BF609 EZ-Board
 * 	Connection details :  N/A
 * 	Guidelines for change: N/A
 */

#ifndef DDR2INIT_H_
#define DDR2INIT_H_
#include<ccblkfn.h>

#if __CCESVERSION__ == 0x01000000
#include "dmc_cces_100_comp.h"
#endif

#define NOP asm volatile("nop;")


/*
 * For the case,  tDD2CLK = 1/250 MHz = 4ns
 * CAS Latency = 4
 * tRCD = 15/4=  4 (3.75)
 * tRP = 15/4 = 4 (3.75)
 * tRC = 60/4 = 15
 * tRRD = 10/4 = 3 (2.5)
 * tRAS = 45/4 = 12 (11.25)
 * tRFC = 127.5/4 = 32(31.875) = 0x20
 * tFAW = 50/4 =  13(12.5)
 * tRTP = 7.5/4 = 2(1.85)
 * tWR = 15/4 = 4 (3.75)
 * tWTR = 7.5/4 = 2 (1.85)
 * tCKE = 3
 * tMRD = 2
 * tXP = 2
 * RDIV = ((f DDCLK X t REFI)) - (t RAS + t RP)
 * RDIV = (250 X 10^6 x 7.8 x 10^-6) - (12 + 4)
 * RDIV = 1950-16 = 1934 = 0x78e
 */

/* DDR2 Init function call declaration */
void DDR2_Init(void);

/* initialize DDR2 using certain CGU values */
void DDR2_Init_wCgu(int iMultiplier, int iCCLKSel, int iDDCLKSel);

/* CLKIN on EZ-Board 25MHz */
#define CLKINMHz 25

/* DDR2 memory specification in DCLK cycles */
#define DDR2_CL  15
#define DDR2_RCD 15
#define DDR2_RP  15
#define DDR2_RC  55
#define DDR2_RRD 10
#define DDR2_RAS 40
#define DDR2_RFC 127.5
#define DDR2_FAW 50
#define DDR2_RTP 7.5
#define DDR2_WR  15
#define DDR2_WTR 7.5
#define DDR2_TREFIus 7.8

/*
 * DDR2 Definitions
 * All numbers are in DCLK cycles
 * CFG bits - Configures SDRAM device specific parameters
 */
#define DMC_IFWID16          		(BITM_DMC_CFG_IFWID & (2 << BITP_DMC_CFG_IFWID))    /* 16 bit interface width */
#define DMC_SDRSIZE1G  			(BITM_DMC_CFG_SDRSIZE & (4 << BITP_DMC_CFG_SDRSIZE))	/* 1Gb SDRAM size */
#define DMC_SDRWID16              (BITM_DMC_CFG_SDRWID & (2 << BITP_DMC_CFG_SDRWID))  	/* 16 bit SDRAM width */
#define DMC_EXTBANK1              (BITM_DMC_CFG_EXTBANK & (0 << BITP_DMC_CFG_EXTBANK))  /* 1 external bank */

/* TR0 bits - SDRAM device timing register 0 */
#define DMC_TRCD       			(BITM_DMC_TR0_TRCD & (uiRCD << BITP_DMC_TR0_TRCD))	/* RAS to CAS */
#define DMC_TWTR 				(BITM_DMC_TR0_TWTR & (uiWTR << BITP_DMC_TR0_TWTR))	/* Write to Read */
#define DMC_TRP					(BITM_DMC_TR0_TRP &  (uiRP << BITP_DMC_TR0_TRP))	/* Precharge to Active */
#define DMC_TRAS				(BITM_DMC_TR0_TRAS & (uiRAS << BITP_DMC_TR0_TRAS)) 	/* Active to Precharge */
#define DMC_TRC					(BITM_DMC_TR0_TRC &  (uiRC << BITP_DMC_TR0_TRC)) 	/* Active to Active time same bank */
#define DMC_TMRD2	(BITM_DMC_TR0_TMRD & (2 << BITP_DMC_TR0_TMRD))					/* Mode register set to active */

/* TR1 bits - SDRAM device timing register 1 */
#define DMC_TRRD				(BITM_DMC_TR1_TRRD & (uiRRD << BITP_DMC_TR1_TRRD))	/* Active to Active different bank */
#define DMC_TRFC				(BITM_DMC_TR1_TRFC & (uiRFC << BITP_DMC_TR1_TRFC)) 	/* Refresh to Active delay */
#define DMC_TREFVAL	            (BITM_DMC_TR1_TREF & (uiREFVAL << BITP_DMC_TR1_TREF))/* Refresh Interval */

/* TR2 bits - SDRAM device timing register 2 */
#define DMC_TCKE3	(BITM_DMC_TR2_TCKE & (3 << BITP_DMC_TR2_TCKE))				/* CKE min pulse width */
#define DMC_TXP2	(BITM_DMC_TR2_TXP &	 (2 << BITP_DMC_TR2_TXP))				/* Exit PD to next valid command */
#define DMC_TRTP				(BITM_DMC_TR2_TRTP & (5 <<BITP_DMC_TR2_TRTP))	/* Internal Read to Precharge time */
#define DMC_TFAW 				(BITM_DMC_TR2_TFAW & (uiFAW << BITP_DMC_TR2_TFAW))	/* Four Active Window */

/* MR bits - Mode Register */
#define DMC_TWR   			   	(BITM_DMC_MR_WRRECOV & 	(uiWR << BITP_DMC_MR_WRRECOV )) /* Write Recovery */
#define DMC_TCL 				(BITM_DMC_MR_CL & 		(uiCL << BITP_DMC_MR_CL))		/* CAS Latency */
#define DMC_BLEN4	(BITM_DMC_MR_BLEN & 	(2 << BITP_DMC_MR_BLEN))					/* Burst Length */
#define DMC_BLEN8	(BITM_DMC_MR_BLEN & 	(3 << BITP_DMC_MR_BLEN))					/* Burst Length */

/* EMR1 bits - Extended Mode Register 1 */
#define DMC_AL3	(BITM_DMC_EMR1_AL & (3 << BITP_DMC_EMR1_AL))		/* Additive latency */

/* CTL bits - DMC Control Register */
#define DMC_RDTOWR2  (BITM_DMC_CTL_RDTOWR & (4 << BITP_DMC_CTL_RDTOWR)) /* Read to Write */

/* DDR0 register configuration */
#define DDR0_CFG		(DMC_IFWID16|DMC_SDRWID16|DMC_SDRSIZE1G|DMC_EXTBANK1)
#define DDR0_TR0 		(DMC_TRCD|DMC_TWTR|DMC_TRP | DMC_TRAS |DMC_TRC |DMC_TMRD2)
#define DDR0_TR1		(DMC_TRRD| DMC_TRFC |DMC_TREFVAL)
#define DDR0_TR2		(DMC_TCKE3| DMC_TRTP|DMC_TXP2|DMC_TFAW|uiWR<<BITP_DMC_TR2_TWR)
#define DDR0_MR			(DMC_TWR | DMC_TCL |DMC_BLEN4)
#define DDR0_EMR1       0//DMC_AL3//3<<BITP_DMC_EMR1_AL
#define DDR0_CTL		(BITM_DMC_CTL_INIT|DMC_RDTOWR2)//|BITM_DMC_CTL_ADDRMODE)//|BITM_DMC_CTL_PPREF)//|BITM_DMC_CTL_ADDRMODE)//)//|BITM_DMC_CTL_ADDRMODE//|)

#endif /* DDR2INIT_H_ */
