/*****************************************************************************
 * DMC_SCB_PRIO_Test_Core0.c
 *****************************************************************************/

/*This code runs two MDMA channels is parallel to read from the DMC. The DMC_PRIOx and DMC_PRIOMSKx registers (ddr_test.c)
 * are programmed to elevate the priority of a particular MDMA channel to increase the DMC throughput for that channel

 */
#include <ccblkfn.h>
#include "adi_initialize.h"
#include "DMC_SCB_PRIO_Test_Core0.h"
#include "ddr_init.h"
#include <cdefBF609.h>
#include <defBF609.h>
#include "time.h"
#include "stdio.h"

#pragma align 32
section("L1_data_a") unsigned char ucIntDestBuff[MAX_SIZE];
section("l2_sram") unsigned char ucIntDestBuff1[MAX_SIZE];


volatile clock_t clock_start;
volatile clock_t clock_stop_mdma0;
volatile clock_t cycles_mdma0;
float throughput_mdma0;

volatile clock_t clock_stop_mdma2;
volatile clock_t cycles_mdma2;
float throughput_mdma2;

int DMA_Done_MDMA0=0;
int DMA_Done_MDMA2=0;

extern void DDR2_Init(void);

void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg);
void MDMA2_Interrupt_Handler(uint32_t iid, void* handlerArg);


//Macro to configure MDMA
#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;


//Macro to wait for MDMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));

//Macro to disable the MDMA
#define DISABLE_DMAs(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\


int main(void)
{
	//DMA work unit size = 2048 bytes
	unsigned int uiBytes=2048;
	unsigned int uiBytes1=2048;

	//Initialize the DMC
	DDR2_Init();

	//Initialize and map the interrupts here
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA0_DST, MDMA0_Interrupt_Handler, NULL, true);
	adi_int_InstallHandler(INTR_MDMA2_DST, MDMA2_Interrupt_Handler, NULL, true);

	//Configure MDMA0 (21-22) to read from DMC
	MDMA_CONFIG(21, SOURCE_DMA_CFG, 0, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,22,DEST_DMA_CFG, ucIntDestBuff, DEST_MSIZE, uiBytes/DEST_MSIZE);

	//Configure MDMA2 (25-26) to read from DMC
	MDMA_CONFIG(25, SOURCE_DMA_CFG, 0x1000000, SOURCE_MSIZE, uiBytes1/SOURCE_MSIZE,26,DEST_DMA_CFG, ucIntDestBuff1, DEST_MSIZE, uiBytes1/DEST_MSIZE);


	//Enable the DMAs here
	*pREG_DMA22_CFG |= 0x1;
	*pREG_DMA26_CFG |= 0x1;
	*pREG_DMA21_CFG |= 0x1;
	*pREG_DMA25_CFG |= 0x1;

	//Start the cycle count
	clock_start = clock();

	//Wait for the DMA done
	while(DMA_Done_MDMA0==0||DMA_Done_MDMA2==0);

	//Stop the cycle counts
	cycles_mdma0=clock_stop_mdma0-clock_start;
	cycles_mdma2=clock_stop_mdma2-clock_start;

	//Disable the DMAs
	DISABLE_DMAs(21,22);
	DISABLE_DMAs(25,26);

	//Calculate the throughput values
	throughput_mdma0=(float)(uiBytes*CCLK_SPEED/cycles_mdma0);
	throughput_mdma2=(float)(uiBytes*CCLK_SPEED/cycles_mdma2);
	printf("\n\n No of bytes=%d, No of cycles for MDMA0 =%d\n\n ",uiBytes,cycles_mdma0);
	printf("\n\n No of bytes=%d, No of cycles for MDMA2 =%d\n\n",uiBytes,cycles_mdma2);
	printf("Throughput MDMA0=%f MB/s, Throughput MDMA2=%f MB/s\n\n",throughput_mdma0, throughput_mdma2);

	return 0;
}


void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count for MDMA0
	clock_stop_mdma0 = clock();

	*pREG_DMA22_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA0 = 1;
}

void MDMA2_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count for MDMA2
	clock_stop_mdma2 = clock();

	*pREG_DMA26_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Done_MDMA2 = 1;
}
