/*****************************************************************************
 * LP DMA Mode.c
 *****************************************************************************
Link Port 0 is receiver
Link Port 1 is transmitter
If lpdir = 0, it indicates link port 0
If lpdir = 1, it indicates link port 1

LPCLOCKDIV_VAL defines the clock div for transmitter
DMA_TRANSFER_CNT defines the data size for transfer
USE_DMA_INTERRUPTS defines if request interrupts should be turned on.
/*****************************************************************************/

#include <ccblkfn.h>
#include <sysreg.h>
#include <cdefBF609.h>
#include <sys/exception.h>
#include <sys/excause.h>
#include <signal.h>
#include <stdio.h>

#include "L2_Max_Throughput.h"

void fill_data_pattern(int lpdir);
void enable_lp_gpio(void);
void program_lpDMA(int lpdir);
void program_lp_clk(int lpdir);
void enable_lpDMA(int lpdir);
void check_lp_request(int lpdir);
void waitDMA_completion(void);
int compare_data(void);
//void LP0_RX_Handler (uint32_t iid, void* handlerArg);
//void LP1_TX_Handler (uint32_t iid, void* handlerArg);
void enable_lpDMA(int lpdir);
//void enable_interrupts(void);

#define DMA_TRANSFER_CNT 8192*2
#define USE_DMA_INTERRUPTS
#define LPCLOCKDIV_VAL 0x44

int lpdir;						   			//0 - rx, 1 - tx. common global to build both tx and rx

#pragma align 32
int lpdata_rx[DMA_TRANSFER_CNT];			//each lp packet is 32 bits.

#pragma align 32
int lpdata_tx[DMA_TRANSFER_CNT];			//each lp packet is 32 bits.

int dma_stat = 1;


extern unsigned char ucIntSrcBuff[MAX_SIZE];


extern unsigned char ucExtDestBuff[MAX_SIZE];


int compare_data()
{
	int dcnt;

	for(dcnt=0;dcnt<DMA_TRANSFER_CNT;dcnt++)
	{
		if(lpdata_rx[dcnt] != lpdata_tx[dcnt])
		{
			printf("\n\nERROR!!!!!!!!\n");
			return 1;
		}

	}
	printf("\n\nSUCCESS!!!!!!!!\n");

	return 0;
}

void fill_data_pattern(int lpdir)
{
	if(lpdir)//fill data only for tx
	{
		int dcnt;
		int data_offset = 0x11111111;
		lpdata_tx[0] = 0x11111111;

		for(dcnt=1;dcnt<DMA_TRANSFER_CNT;dcnt++)
		{
			lpdata_tx[dcnt] = lpdata_tx[dcnt-1] + data_offset;
		}
	}
}

void enable_lp_gpio()
{
	*pREG_PORTA_FER = 0xFFFF;		//enables both lp: 0,1, all signals
	*pREG_PORTB_FER = 0xFFFF;		//enables both lp: 0,1, all signals

	*pREG_PORTA_MUX = 0xAAAAAAAA;	//enables both lp: 0,1, all signals
	*pREG_PORTB_MUX = 0xAAAAAAAA;	//enables both lp: 0,1, all signals
}


void program_lpDMA(int lpdir)
{
	if(lpdir)
	{
		*pREG_LP1_STAT = (BITM_LP_STAT_LRRQ | BITM_LP_STAT_LTRQ);

		//rx dma setup
		*pREG_DMA14_ADDRSTART	= ucIntSrcBuff ;
		*pREG_DMA14_XCNT 		= DMA_TRANSFER_CNT/32;
		*pREG_DMA14_XMOD		= 32;
		*pREG_DMA14_CFG   		= ENUM_DMA_CFG_AUTO    |//stop mode
								  ENUM_DMA_CFG_XCNT_INT|//interrupt
								  ENUM_DMA_CFG_PSIZE04 |//psize 1 byte
								  ENUM_DMA_CFG_MSIZE32 |//msize 4 bytes
								  ENUM_DMA_CFG_READ;    //memory read

		*pREG_LP1_CTL 			= BITM_LP_CTL_TRAN | BITM_LP_CTL_RRQMSK;    //transfer direction tx
	}

	else
	{
		*pREG_LP0_STAT = (BITM_LP_STAT_LRRQ | BITM_LP_STAT_LTRQ);

		//rx dma setup
		*pREG_DMA13_ADDRSTART	= ucExtDestBuff;
		*pREG_DMA13_XCNT 		= DMA_TRANSFER_CNT/32;
		*pREG_DMA13_XMOD		= 32;
		*pREG_DMA13_CFG   		= ENUM_DMA_CFG_AUTO    |//stop mode
								  ENUM_DMA_CFG_XCNT_INT|//interrupt
								  ENUM_DMA_CFG_PSIZE04 |//psize 1 byte
								  ENUM_DMA_CFG_MSIZE32 |//msize 4 bytes
								  ENUM_DMA_CFG_WRITE;   //memory write

		//lp setup
		*pREG_LP0_CTL 			= 0x0 | BITM_LP_CTL_TRQMSK;//transfer direction rx
	}


}

void program_lp_clk(int lpdir)
{
	if(lpdir)
	{
		#ifdef	LOWER_LPCLK_SPEED
			*pREG_LP1_DIV = 8;//LPCLOCKDIV_VAL;							//program clk only for tx
		#else
			*pREG_LP1_DIV = 1;//LPCLOCKDIV_VAL;							//program clk only for tx
		#endif
	}
}

void enable_lpDMA(int lpdir)
{

	if(lpdir)
	{
		*pREG_DMA14_CFG  		|= BITM_DMA_CFG_EN;		//enable dma
		*pREG_LP1_CTL 			|= BITM_LP_CTL_EN ;		//enable tx
	}


	else
	{
		*pREG_DMA13_CFG  		|= BITM_DMA_CFG_EN;		//enable dma
		*pREG_LP0_CTL 			|= BITM_LP_CTL_EN ;		//enable rx
	}
}

void check_lp_request(int lpdir)
{
	int tx_rqst = 0;
	int rx_rqst = 0;

	if(lpdir)
	{
		while(!tx_rqst)
		{
			tx_rqst = *pREG_LP1_STAT & BITM_LP_STAT_LTRQ;
			//wait for tx rqst from external
		}

		tx_rqst = 0;								//clear flag

		*pREG_LP1_STAT |= BITM_LP_STAT_LRRQ; 		//clear rqst


	}
	else
	{
		while(!rx_rqst)
		{
			rx_rqst = *pREG_LP0_STAT & BITM_LP_STAT_LRRQ;
			//wait for rx rqst from external
		}

		rx_rqst = 0;								//clear flag

		*pREG_LP0_STAT |= BITM_LP_STAT_LRRQ; 		//clear rqstt


	}


}

void waitDMA_completion()
{
#ifndef USE_DMA_INTERRUPTS

	while(dma_stat)
	{
		dma_stat = *pREG_DMA13_STAT & BITM_DMA_STAT_RUN;
	}
#else
	while(dma_stat);

#endif


}



